#!/usr/bin/env python

from email import message_from_file
from vCalendar import parse, iterwrite
import imip_store
import sys

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

# Postfix exit codes.

EX_USAGE        = 64
EX_DATAERR      = 65
EX_NOINPUT      = 66
EX_NOUSER       = 67
EX_NOHOST       = 68
EX_UNAVAILABLE  = 69
EX_SOFTWARE     = 70
EX_OSERR        = 71
EX_OSFILE       = 72
EX_CANTCREAT    = 73
EX_IOERR        = 74
EX_TEMPFAIL     = 75
EX_PROTOCOL     = 76
EX_NOPERM       = 77
EX_CONFIG       = 78

# Permitted iTIP content types.

itip_content_types = [
    "text/calendar",                        # from RFC 6047
    "text/x-vcalendar", "application/ics",  # other possibilities
    ]

def process(f, args):
    msg = message_from_file(f)
    print >>open("/tmp/imip.txt", "a"), "----"
    print >>open("/tmp/imip.txt", "a"), args
    print >>open("/tmp/imip.txt", "a"), "----"
    print >>open("/tmp/imip.txt", "a"), msg.as_string()
    print >>open("/tmp/imip.txt", "a")

    # Handle messages with iTIP parts.

    for part in msg.walk():
        if part.get_content_type() in itip_content_types and \
            part.get_param("method"):

            handle_itip_part(part)

def get_itip_elements(elements):
    d = {}
    for name, attr, value in elements:
        if isinstance(value, list):
            d[name] = attr, get_itip_elements(value)
        else:
            if not d.has_key(name):
                d[name] = []
            d[name].append((attr, value))
    return d

def get_attr_value(d, name, single=True):
    if d.has_key(name):
        values = d[name]
        if isinstance(values, tuple):
            return values
        elif single and len(values) == 1:
            return values[0]
        else:
            return values
    else:
        return None

def get_value(d, name, single=True):
    if d.has_key(name):
        values = d[name]
        if isinstance(values, tuple):
            return values[1]
        elif single and len(values) == 1:
            return values[0][1]
        else:
            return map(lambda x: x[1], values)
    else:
        return None

def handle_itip_part(part):
    method = part.get_param("method")

    f = StringIO(part.get_payload(decode=True))
    doctype, attrs, elements = parse(f, encoding=part.get_content_charset())

    if doctype == "VCALENDAR":
        itip = get_itip_elements(elements)

        if get_value(itip, "METHOD") == method:

            for name, cls in handlers:
                details = get_value(itip, name)

                if details:
                    print >>open("/tmp/imip.txt", "a"), details
                    handler = cls(details)
                    print >>open("/tmp/imip.txt", "a"), "Handling", method, "with", handler, "->", methods[method](handler)
                    methods[method](handler)()

class Handler:
    def __init__(self, details):
        self.details = details
        self.uid = get_value(details, "UID")
        self.sequence = get_value(details, "SEQUENCE")
        self.store = imip_store.FileStore()

    def publish(self):
        pass

    def get_attr_value(self, name, single=True):
        return get_attr_value(self.details, name, single)

    def get_value(self, name, single=True):
        return get_value(self.details, name, single)

class Event(Handler):
    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def refresh(self):
        pass

    def reply(self):
        pass

    def request(self):
        pass

class Freebusy(Handler):
    def reply(self):
        attendees = self.get_value("ATTENDEE", False)
        organiser = self.get_value("ORGANIZER")
        if not attendees and not organiser:
            return

        # Store the reply details for the attendee.

    def request(self):
        attendees = self.get_attr_value("ATTENDEE", False)
        organiser = self.get_attr_value("ORGANIZER")

        if not attendees and not organiser:
            return

        organiser_attr, organiser = organiser

        # Get the details for the attendee.

        out = open("/tmp/imip.txt", "a")
        try:
            w = iterwrite(out)

            calendar = []
            cwrite = calendar.append
            cwrite(("METHOD", {}, "REPLY"))
            cwrite(("VERSION", {}, "2.0"))

            for attendee_attr, attendee in attendees:
                freebusy = self.store.get_freebusy(attendee)
                if freebusy:
                    record = []
                    rwrite = record.append
                    rwrite(("ORGANIZER", organiser_attr, organiser))
                    rwrite(("ATTENDEE", attendee_attr, attendee))
                    rwrite(("UID", {}, self.uid))

                    for start, end in freebusy:
                        rwrite(("FREEBUSY", {}, [start, end]))

                    cwrite(("VFREEBUSY", {}, record))

            # Send a reply with the information.

            w.write("VCALENDAR", {}, calendar)

        finally:
            out.close()

class Journal(Handler):
    def add(self):
        pass

    def cancel(self):
        pass

class Todo(Handler):
    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def refresh(self):
        pass

    def reply(self):
        pass

    def request(self):
        pass

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

methods = {
    "ADD"            : lambda handler: handler.add,
    "CANCEL"         : lambda handler: handler.cancel,
    "COUNTER"        : lambda handler: handler.counter,
    "DECLINECOUNTER" : lambda handler: handler.declinecounter,
    "PUBLISH"        : lambda handler: handler.publish,
    "REFRESH"        : lambda handler: handler.refresh,
    "REPLY"          : lambda handler: handler.reply,
    "REQUEST"        : lambda handler: handler.request,
    }

if __name__ == "__main__":
    process(sys.stdin, sys.argv[1:])

# vim: tabstop=4 expandtab shiftwidth=4
